"""
Define generators for scan info publishing in Redis
"""

import enum
import logging
from bliss.scanning import scan_meta
from ..writer import scan_info_generators as writer_info_generators


logger = logging.getLogger(__name__)


def initialize(force=False):
    """
    Initialize the scan info generators

    :param bool force: re-initialize when already done
    """
    # Add custom categories
    generators = scan_meta.get_user_scan_meta()
    used_categories = writer_info_generators.CATEGORIES
    try:
        for attr in used_categories:
            getattr(generators, attr.lower())
        if force:
            raise AttributeError
    except AttributeError:
        cats = [m.name for m in scan_meta.CATEGORIES] + used_categories
        scan_meta.CATEGORIES = enum.Enum(scan_meta.CATEGORIES.__name__, list(set(cats)))
        scan_meta.USER_SCAN_META = None
        # Register the generators
        register_generators()


def register_generators():
    """
    Create the scan info generators
    """
    # Generators are called at the start of the scan:
    #   bliss.scanning.scan.Scan.__init__
    # and at the end of the scan
    #   run bliss.scanning.scan.Scan.run (cleanup section)
    #
    # The generator 'instrument.positioners' is an exception.
    # It is only called at the beginning of the scan by
    # removing it before calling the generators a second time.
    logger.debug('Create scan info generators')
    generators = scan_meta.get_user_scan_meta()
    generators.clear()
    # Add all genrators:
    writer_info_generators.register_generators(generators)
